#ifndef _GUI_H
#define _GUI_H

class Poll;
class ParallelPortIO;
class Simulator;

class GUI {
  public:
  GUI();
  ~GUI();
  void update(ParallelPortIO *ppio, Simulator *s);
  void incrementPoll();
  
  private:
  void loadGraphics();
  void prepareTachometer();
  void prepareSpeedometer();
  void prepareGraph();

  double calculateGraphData(ParallelPortIO *ppio);  
  double calculateTachometer(Poll *sequence);
  double calculateSpeedometer(Poll *sequence);
  double calculatePowerOutput();
  double calculateAccelerometer();

  double interpolate(double *data, double howlongago);
  void averageSmooth(double *srcdata, double *destdata, int width);  
 
  void drawTachometer();
  void drawSpeedometer();
  void drawPowerOverTimeGraph();
  void drawAccelerationOverTimeGraph();
  void drawVelocityOverTimeGraph();
  void drawPowerOverRPMGraph();
  void drawGeneratedDynamoGraph();
  void drawGraph(double *xdata, double *ydata, BITMAP *xaxislabel, BITMAP *yaxislabel, 
    bool XAXISFLIP,
    double XAXISSTART, double XAXISSIZE, int XAXISINTERVAL,
    double YAXISSTART, double YAXISSIZE, int YAXISINTERVAL); 

  void conductDynamometerRun();
  
  void resetGraphData();
  void resetGeneratedGraphData();

  void saveDynamoGraph();

  bool loadDynamoGraph();

  bool obtainString(char *message, int xlocation, int ylocation);
  void displayMessage(char *message, int xlocation, int ylocation);

  double lastupdate;
  int currentscreen;
  int currentscreenstate;
  int lastpolls;
  int numpolls;
  double lastsecond;

  int curfilename;

  BITMAP *doublebuffer;
  BITMAP *logo;
  BITMAP *bgcarimg;
  BITMAP *carwireframe;
  BITMAP *optionmenu;
  BITMAP *tachometer;
  BITMAP *speedometer;
  BITMAP *graphbmp;
  BITMAP *numbers[10];
  BITMAP *numbers20[13];
  BITMAP *needle;
  BITMAP *whiteline;
  BITMAP *smallredline;
  BITMAP *bigredline;
  BITMAP *rpmx1000;
  BITMAP *kmperh;
  BITMAP *powerhplabel;
  BITMAP *enginespeedlabel;
  BITMAP *enginespeedxlabel;
  BITMAP *poweroutputhp;
  BITMAP *time;
  BITMAP *accelkmhpersec;
  BITMAP *speedkmh;
  
  double *graphpowerdata;
  double *graphpowerdatasmoothened;
  double *graphpowerdataallforces;
  double *graphtimestampdata;
  double *graphdistancedata;
  double *graphspeeddata;
  double *graphspeeddatasmoothened;
  double *graphacceldata;
  double *graphacceldatasmoothened;
  double *graphrpmdata;
  int graphdataindex;

  double *generatedgraphx;
  double *generatedgraphy;
  double *generatedgraphspeed;
  double *generatedgraphdistance;
  
  char *userinput;
  bool menukeyboardcontrol;

  //frictional force correction for dynamometer
  double friction_speed;
  double friction_timestamp;
  double friction_force;

};

#endif
