#include <allegro.h>
#include <string.h>
#include <stdio.h>

#include "globals.h"

#include "simulate.h"
#include "parport.h"
#include "poll.h"

Simulator::Simulator() {

  state=1;
  rpm=800;
  speed=0;
  gear=1;
  eventtimestamp=((double)clock())/CLOCKS_PER_SEC;
  tachpulsemissedlastpoll=0.0;
  speedpulsemissedlastpoll=0.0;
 
}

void Simulator::update(ParallelPortIO *ppio) {
 
  double deltatime=((double)clock())/CLOCKS_PER_SEC-eventtimestamp;
  float deltarpm;
  float deltaspeed;
  int newreading;

  if (!DEBUGMODE) {

  if (state==0) {
    deltarpm=-1000.0;
    deltaspeed=-50.0;
  }
  else if (state==1) {
    deltarpm=1000.0;
    deltaspeed=20.0;
  }
  else if (state==2) {
    deltarpm=800.0; 
    deltaspeed=16.0;
  }
  else if (state==3) {
    deltarpm=600.0;
    deltaspeed=12.0;
  }
  else if (state==4) {
    deltarpm=400.0;
    deltaspeed=8.0;
  }
  else if (state==5) {
    deltarpm=200.0;
    deltaspeed=4.0;
  }

  rpm+=deltarpm*deltatime;
  speed+=deltaspeed*deltatime;

  if (rpm<700) rpm=700;
  
  }

  eventtimestamp=((double)clock())/CLOCKS_PER_SEC;

  //create new poll with eventtimestamp and approximate reading for tach  
  newreading=(int)((rpm*2.0*deltatime/60.0)+tachpulsemissedlastpoll);
  tachpulsemissedlastpoll=(rpm*2.0*deltatime)/60.0+tachpulsemissedlastpoll-newreading;
  ppio->addPoll(TACHBUSADDR,eventtimestamp,newreading);

  //create new poll with eventtimestamp and approximate reading for speedo  
  newreading=(int)((speed*SPEEDPULSESPERKM*deltatime/3600.0)+speedpulsemissedlastpoll);
  speedpulsemissedlastpoll=(speed*SPEEDPULSESPERKM*deltatime)/3600.0+speedpulsemissedlastpoll-newreading;
  ppio->addPoll(SPEEDBUSADDR,eventtimestamp,newreading);

  //state change
  if (!DEBUGMODE) {

  if (state!=0) {
    if (rpm>3500) {
      rpm=2000;
      state++;
      gear=state;
    }
  
    if (state>5)
      state=0;
  }
  else if (state==0) {
    if (speed<=10)
      state=1;
  }
  }

}

void Simulator::incrementSpeed(float amount) {
  speed+=amount;
}

void Simulator::incrementRPM(float amount) {
  rpm+=amount;
}
